import sys
import pytest
from mock import Mock

from virgotools.cm_lib import Char
from virgotools import interface_acl


CM_NAME = 'mock_acl'

mock_cm_send = Mock()
mock_cm_send_receive = Mock(return_value=['a', 'b'])
mock_cm_lib = Mock(cm_send=mock_cm_send, cm_send_receive=mock_cm_send_receive)

@pytest.fixture
def server():
    sys.modules['virgotools.cm_lib'] = mock_cm_lib
    yield interface_acl.AclServer(CM_NAME + '.cfg')
    # clean up, might need to test the real thing later
    del sys.modules['virgotools.cm_lib']


def test_parsing(server):
    assert hasattr(server, 'CONST')
    assert hasattr(server, 'SINE')
    assert hasattr(server, 'LINE')
    assert hasattr(server, 'COLOR')


def test_cm(server):
    server.cm_send('sometype', 5, 3.14, 'somestr', Char('a'))
    mock_cm_send.assert_called_with(CM_NAME, 'sometype', 5, 3.14, 'somestr', Char('a'))

    server.CONST = 5
    mock_cm_send.assert_called_with(CM_NAME, 'AcConstChSet', 'CONST', 5.0)

    server.SUM.SINE = 1.23
    mock_cm_send.assert_called_with(CM_NAME, 'AcSumChSet', 'SUM', 1.0, 1.23, 'SINE')


def test_paramget(server):
    server.CONST.AcParamGet()
    mock_cm_send_receive.assert_called_with('AcParam', 2, CM_NAME, 'AcParamGet', 'CONST')

def test_float_math(server):
    # weight of 2*3/4 should evaluate to 1.5
    assert server.math_float._line.weights[0] == 1.5


